@@ -2,11 +2,11 @@ import time
 import spidev
 import RPi.GPIO as GPIO
 
-__version__ = '0.0.2'
+__version__ = '0.0.3'
 
 
 class APA102():
-    def __init__(self, count=1, gpio_data=14, gpio_clock=15, gpio_cs=None, brightness=1.0, force_gpio=False):
+    def __init__(self, count=1, gpio_data=14, gpio_clock=15, gpio_cs=None, brightness=1.0, force_gpio=False, invert=False, spi_max_speed_hz=1000000):
         """Initialise an APA102 device.
 
         Will use SPI if it's available on the specified data/clock pins.
@@ -22,16 +22,19 @@ class APA102():
         self._gpio_clock = gpio_clock
         self._gpio_data = gpio_data
         self._gpio_cs = gpio_cs
+        self._invert = invert
+
+        if invert:
+            # TODO Add invert support for SPI
+            force_gpio = True
 
         self._gpio = None
         self._spi = None
         self._brightness = brightness
 
-        self._sof_length = 8 * 8  # SOF bytes
-        self._eof_length = 8 * 4  # EOF bytes
+        self._sof_length = 4  # SOF bytes
+        self._eof_length = 4  # EOF bytes
         buffer_length = count * 4
-        buffer_length += self._sof_length
-        buffer_length += self._eof_length
 
         self._buf = []
 
@@ -43,29 +46,41 @@ class APA102():
         for _ in range(self._eof_length):
             self._buf.append(0b11111111)
 
-        if not force_gpio and gpio_data == 10 and gpio_clock == 11 and gpio_cs in (7, 8):
-            self._spi = spidev.SpiDev(0, [8, 7].index(gpio_cs))
+        if not force_gpio and gpio_data == 10 and gpio_clock == 11 and gpio_cs in (None, 7, 8):
+            cs_channel = 0
+            if gpio_cs is not None:
+                cs_channel = [8, 7].index(gpio_cs)
+            self._spi = spidev.SpiDev(0, cs_channel)
+            self._spi.max_speed_hz = spi_max_speed_hz
+            if gpio_cs is None:
+                self._spi.no_cs = True
             self._gpio_cs = None
 
-        elif not force_gpio and gpio_data == 20 and gpio_clock == 21 and gpio_cs in (18, 17, 16):
-            self._spi = spidev.SpiDev(0, [18, 17, 16].index(gpio_cs))
+        elif not force_gpio and gpio_data == 20 and gpio_clock == 21 and gpio_cs in (None, 18, 17, 16):
+            cs_channel = 0
+            if gpio_cs is not None:
+                cs_channel = [18, 17, 16].index(gpio_cs)
+            self._spi = spidev.SpiDev(0, cs_channel)
+            self._spi.max_speed_hz = spi_max_speed_hz
+            if gpio_cs is None:
+                self._spi.no_cs = True
             self._gpio_cs = None
 
         else:
             self._gpio = GPIO
             self._gpio.setmode(GPIO.BCM)
-            self._gpio.setup([gpio_data, gpio_clock], GPIO.OUT)
+            self._gpio.setup(gpio_data, GPIO.OUT, initial=1 if self._invert else 0)
+            self._gpio.setup(gpio_clock, GPIO.OUT, initial=1 if self._invert else 0)
             if self._gpio_cs is not None:
                 self._gpio.setup(self._gpio_cs, GPIO.OUT)
 
-
     def _write_byte(self, byte):
         for _ in range(8):
-            self._gpio.output(self._gpio_data, byte & 0x80)
-            self._gpio.output(self._gpio_clock, 1)
+            self._gpio.output(self._gpio_data, not (byte & 0x80) if self._invert else (byte & 0x80))
+            self._gpio.output(self._gpio_clock, 0 if self._invert else 1)
             time.sleep(0)
             byte <<= 1
-            self._gpio.output(self._gpio_clock, 0)
+            self._gpio.output(self._gpio_clock, 1 if self._invert else 0)
             time.sleep(0)
 
     def set_pixel(self, x, r, g, b):
@@ -78,7 +93,7 @@ class APA102():
 
         """
         offset = self._sof_length + (x * 4) + 1
-        self._buf[offset:offset + 3] = [r, g, b]
+        self._buf[offset:offset + 3] = [b, g, r]
 
     def set_brightness(self, x, brightness):
         """Set global brightness of a single pixel
@@ -88,7 +103,7 @@ class APA102():
 
         """
         offset = self._sof_length + (x * 4)
-        self._buf[offset] = int(31 * brightness)
+        self._buf[offset] = 0b11100000 | int(31 * brightness)
 
     def show(self):
         """Display the buffer
